require 'uu/applog/application_log/severity'
require 'uu/applog/application_log/application_log_create'
require 'uu/applog/application_log/application_log_add_record'
require 'uu/applog/application_log/application_log_get_record_list'
require 'uu/applog/application_log/application_log_record_attributes'
require 'uu/applog/application_log/application_log_set_attributes'
require 'uu/applog/application_log/application_log_attributes'

require 'uu/os/rest/remote_client'
require 'uu/os/rest/result_list'
require 'uu/os/qos/qos_handler'
require 'uu/os/uesuri'

module UU
  # AppLog module
  module AppLog
    # Application log is a service designed for application debugging and monitoring.
    module ApplicationLog
      extend self

      # URL path of the corresponding REST service.
      PATH = 'uu/applog/ApplicationLog'

      # Creates a new application log in the specified uuApplication.
      # @param [String, UU::OS::UESURI] uuapp_uri uuApplication UESURI.
      # @param [ApplicationLogCreate] applog_attributes
      #   DTO containing attributes of the new application log.
      # @return [UU:OS::UESURI] UESURI of the new application log.
      def create(uuapp_uri, applog_attributes = nil)
        svc = UU::OS::REST::RemoteClient.new(ApplicationLog)
        payload = ApplicationLog::ApplicationLogCreate.new(applog_attributes).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', uuapp_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Get application log configuration attributes.
      #
      # @param [String, UU::OS::UESURI] applog_uri UESURI of the application log
      # @return [ApplicationLogAttributes] application log attributes with configuration
      def get_attributes(applog_uri)
        svc = UU::OS::REST::RemoteClient.new(ApplicationLog)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', applog_uri)
          return ApplicationLog::ApplicationLogAttributes.new(res)
        end
      end

      # Set configuration attributes of the application log.
      #
      # @param [String, UU::OS::UESURI] applog_uri UESURI of the application log
      # @param [ApplicationLogSetAttributes] applog_set_attributes DTO containing changed attributes of application log
      # @return [UU::OS::UESURI] UESURI of application log
      def set_attributes(applog_uri, applog_set_attributes = nil)
        svc = UU::OS::REST::RemoteClient.new(ApplicationLog)
        payload = ApplicationLog::ApplicationLogSetAttributes.new(applog_set_attributes)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', applog_uri, payload.to_json)
          return UU::OS::UESURI.new(res)
        end
      end

      # Adds a new record to the specified application log.
      #
      # @example
      #   UU::AppLog::ApplicationLog.add_record(applog_uri,
      #     creation_time: Time.now,
      #     logger_code: "CdsGb::Cmd::CommentCreateController",
      #     severity: "INFO",
      #     message: "Comment created."
      #   )
      #
      # @param applog_uri [String, UU::OS::UESURI] UESURI of the application
      #   log to add record to.
      # @param log_record [ApplicationLogAddRecord] Attributes of the new
      #   application log record.
      # @param [Object] auth_token_aware Optional object that is aware of
      #   application log authentication token, i.e. responding to
      #   +uu_applog_auth_token+ method. It is used for authentication
      #   optimization - authentication token is generated on server-side
      #   during first call and then it is used to minimize response time of
      #   the next calls.
      # @return [UU::OS::UESURI] UESURI of the new application log record.
      def add_record(applog_uri, log_record, auth_token_aware=nil)
        svc = UU::OS::REST::RemoteClient.new(ApplicationLog)
        payload = ApplicationLog::ApplicationLogAddRecord.new(log_record).to_json

        if auth_token_aware.respond_to?:uu_applog_auth_token
          svc.add_header('uu-applog-token', auth_token_aware.uu_applog_auth_token) unless auth_token_aware.nil?
        end
        # QoS is not checked for this operation (records shall be created
        # regardless of QoS limits so that the related issues can be
        # troubleshooted)
        raw_res = svc.raw_post('addRecord', applog_uri, payload)
        res = svc.send(:process_result, raw_res) #TODO change send to normal method call if become public

        # set token to the auth token aware class only if there is some
        if auth_token_aware.respond_to?:uu_applog_auth_token
          token = raw_res[2][:uu_applog_token]
          auth_token_aware.uu_applog_auth_token=token unless auth_token_aware.nil? || token.nil? || token.empty?
        end

        return UU::OS::UESURI.new(res)
      end

      # Returns a list of records from the specified application log. The list can
      # optionally be limited and/or ordered (see the +criteria+ parameter).
      # @param [String, UU::OS::UESURI] applog_uri Application log UESURI
      # @param [ApplicationLogGetRecordList] criteria
      #   DTO containing criteria limiting and ordering the record list.
      # @return [Array] Array of application log records according to the specified
      #   criteria.
      def get_record_list(applog_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(ApplicationLog)
        dto = ApplicationLog::ApplicationLogGetRecordList.new(criteria)
        svc.add_parameter('timeFrom', dto.time_from)
        svc.add_parameter('timeTo', dto.time_to)
        svc.add_parameter('offset', dto.offset)
        svc.add_parameter('limit', dto.limit)

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getRecordList', applog_uri)
          data = JSON.parse(res, :symbolize_names => true)
          result = data.map { |value| ApplicationLog::ApplicationLogRecordAttributes.new(value)}
          return result
        end
      end

    end
  end
end

